@@ -17,7 +17,9 @@
 package org.jbpm.workbench.pr.client.editors.instance.diagram;
 
 import java.util.List;
+import java.util.Map;
 import java.util.Optional;
+import java.util.stream.Collectors;
 
 import javax.annotation.PostConstruct;
 import javax.enterprise.context.Dependent;
@@ -28,19 +30,24 @@ import javax.inject.Inject;
 import com.google.gwt.user.client.ui.IsWidget;
 import org.jboss.errai.common.client.api.Caller;
 import org.jbpm.workbench.common.client.util.DateUtils;
+import org.jbpm.workbench.common.preferences.ManagePreferences;
 import org.jbpm.workbench.pr.client.editors.instance.ProcessInstanceSummaryAware;
 import org.jbpm.workbench.pr.client.resources.i18n.Constants;
+import org.jbpm.workbench.pr.events.ProcessDiagramExpandEvent;
 import org.jbpm.workbench.pr.events.ProcessInstanceSelectionEvent;
 import org.jbpm.workbench.pr.model.NodeInstanceSummary;
 import org.jbpm.workbench.pr.model.ProcessInstanceDiagramSummary;
+import org.jbpm.workbench.pr.model.ProcessInstanceKey;
 import org.jbpm.workbench.pr.model.ProcessInstanceSummary;
 import org.jbpm.workbench.pr.model.ProcessNodeSummary;
 import org.jbpm.workbench.pr.model.TimerInstanceSummary;
 import org.jbpm.workbench.pr.model.TimerSummary;
 import org.jbpm.workbench.pr.service.ProcessRuntimeDataService;
 import org.kie.api.runtime.process.ProcessInstance;
+import org.kie.workbench.common.widgets.client.popups.alert.AlertPopupView;
 import org.uberfire.client.annotations.WorkbenchPartTitle;
 import org.uberfire.client.annotations.WorkbenchPartView;
+import org.uberfire.client.views.pfly.widgets.ConfirmPopup;
 import org.uberfire.workbench.events.NotificationEvent;
 
 import static java.util.Collections.emptyList;
@@ -72,10 +79,29 @@ public class ProcessInstanceDiagramPresenter implements ProcessInstanceSummaryAw
     @Inject
     private Event<NotificationEvent> notification;
 
+    @Inject
+    protected ManagePreferences preferences;
+
+    private ConfirmPopup confirmPopup;
+
+    private AlertPopupView alertPopup;
+
     @PostConstruct
     public void init() {
         view.setOnProcessNodeSelectedCallback(id -> onDiagramNodeSelected(id));
         view.setOnDiagramNodeSelectionCallback(id -> onDiagramNodeSelected(id));
+        view.setOnDiagramNodeSelectionDoubleClick(id -> onDiagramNodeSelectedByDoubleClick(id));
+        preferences.load();
+    }
+
+    @Inject
+    protected void setConfirmPopup(final ConfirmPopup confirmPopup) {
+        this.confirmPopup = confirmPopup;
+    }
+
+    @Inject
+    protected void setAlertPopup(final AlertPopupView alertPopup){
+        this.alertPopup = alertPopup;
     }
 
     @Override
@@ -112,6 +138,7 @@ public class ProcessInstanceDiagramPresenter implements ProcessInstanceSummaryAw
 
             view.setProcessNodes(processNodes);
 
+            view.setShowOrHideParentAndSubProcessPanelCommand(() -> showOrHideParentAndSubProcessPanel(summary.getParentProcessInstanceSummary(), summary.getSubProcessInstances()));
             nodeInstances = summary.getNodeInstances().stream().sorted(comparing(NodeInstanceSummary::getName, String.CASE_INSENSITIVE_ORDER).thenComparingLong(NodeInstanceSummary::getId)).collect(toList());
 
             nodeInstances.forEach(ni -> {
@@ -124,8 +151,17 @@ public class ProcessInstanceDiagramPresenter implements ProcessInstanceSummaryAw
                 }
             });
 
+            final Map<String, Long> badges = nodeInstances.stream().collect(Collectors.groupingBy(NodeInstanceSummary::getNodeUniqueName, Collectors.counting()));
+            view.setNodeBadges(badges);
+
             view.setNodeInstances(nodeInstances);
 
+            view.showOrHideParentAndSubProcessPanelTriggered();
+            view.setSubProcessInstances(summary.getSubProcessInstances());
+            view.setParentProcessInstance(summary.getParentProcessInstanceSummary());
+
+            view.setParentSelectedCommand(() -> listenParentProcessInstanceSelected());
+
             timerInstances = summary.getTimerInstances().stream().sorted(comparing(TimerInstanceSummary::getName, String.CASE_INSENSITIVE_ORDER).thenComparingLong(TimerInstanceSummary::getId)).collect(toList());
 
             timerInstances.forEach(ti -> {
@@ -140,12 +176,33 @@ public class ProcessInstanceDiagramPresenter implements ProcessInstanceSummaryAw
 
             view.setTimerInstances(timerInstances);
 
+            view.setShowOrHideNodeActionsCommand(() -> showOrHideNodeActions(forLog, processInstance));
+
             timers = summary.getProcessDefinition().getTimers().stream().collect(toList());
 
-            if (forLog || processInstance.getState() != ProcessInstance.STATE_ACTIVE) {
-                view.hideNodeActions();
-            }
-        }).getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey());
+            view.showOrHideNodeActionsTriggered();
+            expandDiagram(processInstance, summary);
+
+            view.onShow();
+        }).getProcessInstanceDiagramSummary(processInstance.getProcessInstanceKey(),
+                                            preferences.getProcessInstanceDiagramCompletedNodeColor(),
+                                            preferences.getProcessInstanceDiagramCompletedNodeBorderColor(),
+                                            preferences.getProcessInstanceDiagramActiveNodeBorderColor());
+    }
+
+    protected void expandDiagram(ProcessInstanceSummary processInstance, ProcessInstanceDiagramSummary summary) {
+        if ((forLog || processInstance.getState() != ProcessInstance.STATE_ACTIVE) && summary.getSubProcessInstances().isEmpty() && summary.getParentProcessInstanceSummary() == null) {
+            view.expandDiagram();
+            view.disableExpandAnchor();
+        }
+    }
+
+    protected void showOrHideParentAndSubProcessPanel(ProcessInstanceSummary parentProcessInstance, List<ProcessInstanceSummary> subProcessInstances) {
+        if (subProcessInstances.isEmpty() && parentProcessInstance == null) {
+            view.hideParentAndSubProcessPanel();
+        } else {
+            view.showParentAndSubProcessPanel();
+        }
     }
 
     public void displayImage(final String svgContent, final String containerId) {
@@ -169,6 +226,30 @@ public class ProcessInstanceDiagramPresenter implements ProcessInstanceSummaryAw
         }
     }
 
+    protected void onDiagramNodeSelectedByDoubleClick(final String nodeId) {
+        List<NodeInstanceSummary> nodeInstanceSummaries = nodeInstances.stream().filter(node -> node.getNodeUniqueName().equals(nodeId)).collect(toList());
+        List<NodeInstanceSummary> nodeInstancesSubProcess = nodeInstanceSummaries.stream().filter(node -> node.getType().equals("SubProcessNode")).collect(toList());
+        if (nodeInstancesSubProcess.size() == 1) {
+            openNavigationConfirmPopup(nodeInstancesSubProcess.get(0));
+        }
+        if (nodeInstancesSubProcess.size() > 1) {
+            alertPopup.alert(constants.AlertPopupTitle(), constants.AlertPopupMessage());
+        }
+        if (nodeInstancesSubProcess.size() < 1) {
+            view.setNodeInstances(nodeInstanceSummaries);
+            view.setTimerInstances(getTimerInstanceForNode(nodeId));
+        }
+        view.setValue(processNodes.stream().filter(node -> node.getUniqueId().equals(nodeId)).findFirst().orElseGet(() -> new ProcessNodeSummary()));
+    }
+
+    protected void openNavigationConfirmPopup(NodeInstanceSummary nodeInstance) {
+        confirmPopup.show(constants.NavigationConfirmTitle(), constants.NavigationConfirmOkButtonText(), constants.NavigationConfirmMessage(),
+                          () -> {
+                              ProcessInstanceKey processInstanceKey = new ProcessInstanceKey(processInstance.getProcessInstanceKey().getServerTemplateId(), processInstance.getProcessInstanceKey().getDeploymentId(), nodeInstance.getReferenceId());
+                              processInstanceEvent.fire(new ProcessInstanceSelectionEvent(processInstanceKey, forLog, true));
+                          });
+    }
+
     protected List<TimerInstanceSummary> getTimerInstanceForNode(final String nodeId) {
         final Optional<TimerSummary> summary = timers.stream().filter(timer -> timer.getUniqueId().equals(nodeId)).findFirst();
         if (summary.isPresent()) {
@@ -209,6 +290,11 @@ public class ProcessInstanceDiagramPresenter implements ProcessInstanceSummaryAw
                                                                     forLog));
     }
 
+    public void listenParentProcessInstanceSelected() {
+        processInstanceEvent.fire(new ProcessInstanceSelectionEvent(new ProcessInstanceKey(processInstance.getServerTemplateId(),
+                                                                                           processInstance.getDeploymentId(), processInstance.getParentId()), false, true));
+    }
+
     public void onNodeInstanceReTrigger(final NodeInstanceSummary node) {
         processService.call((Void) -> {
             notification.fire(new NotificationEvent(constants.NodeInstanceReTriggered(node.getLabel()),
@@ -236,6 +322,10 @@ public class ProcessInstanceDiagramPresenter implements ProcessInstanceSummaryAw
                                    summary);
     }
 
+    public void onShow(){
+        view.onShow();
+    }
+
     @WorkbenchPartTitle
     public String getName() {
         return constants.Diagram();
@@ -250,4 +340,22 @@ public class ProcessInstanceDiagramPresenter implements ProcessInstanceSummaryAw
     public void setProcessService(final Caller<ProcessRuntimeDataService> processService) {
         this.processService = processService;
     }
+
+    public void onProcessDiagramExpandEvent(@Observes ProcessDiagramExpandEvent event) {
+        if (event.isExpand()) {
+            view.hideNodeActions();
+            view.hideParentAndSubProcessPanel();
+        } else {
+            view.showOrHideNodeActionsTriggered();
+            view.showOrHideParentAndSubProcessPanelTriggered();
+        }
+    }
+
+    protected void showOrHideNodeActions(boolean forLog, ProcessInstanceSummary processInstance) {
+        if (forLog || processInstance.getState() != ProcessInstance.STATE_ACTIVE) {
+            view.hideNodeActions();
+        } else {
+            view.showNodeActions();
+        }
+    }
 }
